package securityclassloaderv2;

/**
 * Title:        Eigener Class Loader
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */
import java.util.*;
import java.net.*;
import java.io.*;

public class MeinClassLoader extends ClassLoader {
    // JDK 1.0.x : jeder ClassLoader besitzt einen eigenen Cache
    // private Hashtable cache = new Hashtable();
    private final File myBase;
    private static final boolean verbose = Boolean.getBoolean("Debug");

    public MeinClassLoader(File myBase) {
	if (verbose) { log("MeinClassLoader: constructor, mybase = " +
	    myBase); }
	this.myBase = myBase;
    }

    private byte[] loadClassData(String name)
	throws ClassNotFoundException {
	byte [] buffer;

	if (verbose) { log("MeinClassLoader: loadClassData, name=" +
	    name); }
	File source = null;
	InputStream s = null;
	try {
	    source = new File(myBase, name);
	    s = new FileInputStream(source);
	    buffer = new byte [(int)(source.length())];
	    s.read(buffer);
	}
	catch (Exception e) {
	    System.out.println("Probleme mit " + source);
	    throw new ClassNotFoundException(name);
	}
	return buffer;
    }

    public synchronized Class loadClass(String name, boolean resolve)
	throws ClassNotFoundException {
	if (verbose) { log("MeinClassLoader: loadClass, name=" + name +
	    " resolve=" + resolve); }
	if (name.charAt(0) == '.') {
	    throw new SecurityException("'Absolute' package name");
	}
	if (name.indexOf("..") != -1) {
	    throw new SecurityException("Illegal package name");
	}

	// jdk1.0.x
	// Class c = (Class)(cache.get(name));
        // Test : was passiert, wenn der Cache nicht abgefragt wird
	//Class c=null;// = (findLoadedClass(name));
        Class c = (findLoadedClass(name));

	if (c == null) {
	    try {
		c = findSystemClass(name);
	    }
	    catch (Exception e) {
		try {
		    String path = name.replace('.', File.separatorChar) +
		        ".class";
		    byte data[] = loadClassData(path);
		    c = defineClass(name, data, 0, data.length);
		    // JDK 1.1+ defineClass cached die Klassen
		    // JDK 1.0.x - benutzen des eigenen Caches
		    // cache.put(name, c);
		}
		catch (ClassFormatError ex) {
		    throw new ClassNotFoundException(" Class ist: " + name +
			" Problem ist: " + ex.getMessage());
		}
	    }
	}
	if (resolve) {
	    resolveClass(c);
	}
	return c;
    }

    private void log(String s) {
	System.err.println(s);
    }

    public static void main(String args[]) throws Throwable {
	if (args.length < 2) {
	    System.out.println("MeinClassLoader : fehlendes Argument ");
	    System.exit(1);
	}
	File base = new File(args[0]);

	if (!(base.exists() && base.isDirectory())) {
	    System.out.println("Verzeichnisfehler " + args[0]);
	    System.exit(1);
	}
	ClassLoader cl = new MeinClassLoader(base);
	Class c = cl.loadClass(args[1]);
	Class c1 = cl.loadClass(args[2]);
	if (c1 == c) {
	    System.out.println("Die zwei Klassen sind identisch "+
		"(referenzieren auf das gleiche Objekt); die zweite Klasse  "+
		"wurde somit aus dem Cache geladen.");
	} else {
	    System.out.println("Die zwei Klassen sind unterschiedlich;"+
		" die zweite Klasse wurde somit nicht im Cache gefunden.");
	}
	//Runnable r = (Runnable)(c.newInstance());
	//r.run();

    }
}
