package securityjavarunner;

/**
 * Title:        JavaRunner
 * Description:  Simuliert das Laden einer Klasse
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.lang.reflect.*;

public class JavaRunner implements Runnable {
	final static int numArgs = 1;
	private Object args[];
	private String className;
        private boolean bAusgabe=false;

	JavaRunner(String className, Object args[]) {
	    this.className = className;
	    this.args = args;
	}

	void invokeMain(Class clazz) {
		Class argList[] = new Class[] { String[].class };
		Method mainMethode = null;
		try {
			mainMethode = clazz.getMethod("main", argList);
		} catch (NoSuchMethodException nsme) {
			System.out.println("Es wurde keine main Methode in " + clazz.getName()+" gefunden");
			System.exit(-1);
		}
		try {
			mainMethode.invoke(null, args);
		} catch (Exception e) {
			Throwable t;
			if (e instanceof InvocationTargetException)
				t = ((InvocationTargetException) e)
										.getTargetException();
			else t = e;
			System.out.println("Der Methodenaufruf endete mit einer " + t);
			t.printStackTrace();
		}
	}

	void invokeAusgabe(Class clazz) {
		Class argList[] = new Class[] { String[].class };
		Method ausgabeMethode=null;
		try {
			//mainMethode = clazz.getMethod("main", argList);
                        ausgabeMethode = clazz.getMethod("ausgabe",null);
                        System.out.println("Methode Ausgabe: "+ausgabeMethode.getName());
                        bAusgabe=true;
		} catch (NoSuchMethodException nsme) {
			System.out.println("Es wurde keine 'ausgabe' Methode in " + clazz.getName()+" gefunden");
			bAusgabe=false;
		}
		try {
			//mainMethode.invoke(null, args);
                        if (bAusgabe) {
                          Class argumente[] = new Class[ausgabeMethode.getParameterTypes().length];
                          for (int i=0;i<ausgabeMethode.getParameterTypes().length; i++)
                            argumente[i] = ausgabeMethode.getParameterTypes()[i];
                          System.out.println("Anzahl Parameter : "+ausgabeMethode.getParameterTypes().length);
                          ausgabeMethode.invoke(null, argumente);
                        }
		} catch (Exception e) {
			Throwable t;
			if (e instanceof InvocationTargetException)
				t = ((InvocationTargetException) e)
										.getTargetException();
			else t = e;
			System.out.println("Der Methodenaufruf endete mit einer " + t);
			t.printStackTrace();
		}
	}

	public void run() {
		Class target = null;
		try {
			target = Class.forName(className);
			invokeMain(target);
                        invokeAusgabe(target);
		} catch (ClassNotFoundException cnfe) {
			System.out.println("Die Klasse " + className+" kann nicht geladen werden!");
		}
	}

	static Object[] getArgs(String args[]) {
		String passArgs[] = new String[args.length - numArgs];
		for (int i = numArgs; i < args.length; i++)
			passArgs[i - numArgs] = args[i];

		Object wrapArgs[] = new Object[1];
		wrapArgs[0] = passArgs;
		return wrapArgs;
	}

	public static void main(String args[]) {
                System.out.println("System ClassLoader "+ClassLoader.getSystemClassLoader().getClass().getName());
		try {
		if (args.length < 1) {
			System.err.println("usage:  JavaRunner classfile");
			System.exit(-1);
		}
		ThreadGroup tg = new ThreadGroup("JavaRunner Threadgroup");
		Thread t = new Thread(tg,
				new JavaRunner(args[0], getArgs(args)));
		t.start();
		try {
			t.join();
		} catch (InterruptedException ie) {
			System.out.println("Der JavaRunner Thread wurde unterbrochen");
		}
		} catch (Exception e) {
		    e.printStackTrace();
		}
	}
}
