package securitymanagerzusammenfassung;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.util.*;
import java.net.*;
import java.io.*;
import java.security.*;
import java.math.*;

public class SampleClassLoader extends ClassLoader {
    private File myBase;
    private static final boolean verbose = Boolean.getBoolean ("Debug");

    public SampleClassLoader (File mb) {
	myBase = mb;
	log ("SampleClassLoader: constructor, mb = " + mb);
    }

    private byte [] loadClassData (String name)
	throws ClassNotFoundException {
	byte [] buffer;

	log ("SampleClassLoader: loadClassData, name=" + name);
	File source = null;
	InputStream s = null;
	try {
	    source = new File (myBase, name);
	    s = new FileInputStream (source);
	    buffer = new byte [(int)(source.length ())];
	    s.read (buffer, 0, 8); // read 8 bytes
	    DataInputStream in =
		new DataInputStream (new ByteArrayInputStream (buffer));
	    int magic = in.readInt ();
	    int sum = in.readInt ();
	    if (magic == 0xadc0ffee) {
		// signed, lies alles in den Buffer,
		if (verbose) {
		    log ("Classfile " + source.getName () +
		        " ist mit Quersumme versehen");
		}
		s.read (buffer);
		int sumRead = Sum.calculate (buffer);
		if (sum != sumRead) {
		    if (verbose) { log ("Quersumme stimmt nicht!"); }
		    throw new ClassNotFoundException ("Classfile " +
		        source.getName () +
 				" besitzt eine fehlerhafte Quersumme");
                }
            } else {
		// Classfile ohne Checksumme
		// lies den Rest
		if (verbose) {
		    log ("Classfile " + source.getName () +
		        " ist ohne Quersumme");
		}
		s.read (buffer, 8, buffer.length - 8);
	    }
	}
	catch (Exception e) {
	    System.out.println ("Probleme mit " + source);
	    throw new ClassNotFoundException (name);
	}
	return buffer;
    }
    public synchronized Class loadClass (String name, boolean resolve)
	throws ClassNotFoundException {
	log ("SampleClassLoader: loadClass, name=" + name +
		" resolve=" + resolve);
	if (name.charAt (0) == '.') {
	    throw new SecurityException ("'Absolute' package name");
	}
	if (name.indexOf ("..") != -1) {
	    throw new SecurityException ("Illegal package name");
	}

	Class c = findLoadedClass (name);

	if (c == null) {
	    try {
		c = findSystemClass (name);
	    }
	    catch (Exception e) {
		log ("SampleClassLoader: class ist non-system");
		try {
		    String path = name.replace
		        ('.', File.separatorChar) + ".class";
		    byte [] data = loadClassData (path);
		    c = defineClass (name, data, 0, data.length);
		}
		catch (ClassFormatError ex) {
		    throw new ClassNotFoundException (name);
		}
	    }
	}
	if (resolve) {
	    resolveClass (c);
	}
	return c;
    }
    private void log (String s) {
	if (verbose) {
	    System.err.println (s);
	}
}
    public static void main (String args[]) throws Throwable {
	if (args.length < 2) {
	    System.out.println (
		"SampleClassLoader  ");
	    System.exit (1);
	}
	File base = new File (args[0]);

	if (!(base.exists () && base.isDirectory ())) {
	    System.out.println ("Verzeichnisfehler " + args[0]);
	    System.exit (1);
	}

	System.setSecurityManager (new SampleSecurityManager ());
	ClassLoader cl = new SampleClassLoader (base);

	Class c = cl.loadClass (args[1]);
	Runnable r = (Runnable)(c.newInstance ());
	r.run ();
    }
}
